@@ -1,4 +1,3 @@
-#!/usr/local/bin/python3.8
 # -*- coding: utf-8 -*-
 #
 # clienthelpers.py - helper functions for xgps and test_maidenhead
@@ -10,6 +9,9 @@
 # This file is Copyright 2019 by the GPSD project
 # SPDX-License-Identifier: BSD-2-Clause
 """GPSd client helpers submodule."""
+
+from __future__ import absolute_import  # Ensure Python2 behaves like Python 3
+import gps
 import math
 import os
 
@@ -762,9 +764,9 @@ def deg_to_str(fmt, degrees):
     if _non_finite(degrees):
         return ''
 
-    if degrees >= 360:
+    if 360 <= degrees:
         degrees -= 360
-    if not math.fabs(degrees) <= 360:
+    if not 360 >= math.fabs(degrees):
         return ''
 
     if fmt is deg_dd:
@@ -784,8 +786,11 @@ def deg_to_str(fmt, degrees):
 
 def gpsd_units():
     """Deduce a set of units from locale and environment."""
-    unit_lookup = {'imperial': imperial,
+    unit_lookup = {'i': imperial,
+                   'imperial': imperial,
+                   'm': metric,
                    'metric': metric,
+                   'n': nautical,
                    'nautical': nautical}
     if 'GPSD_UNITS' in os.environ:
         store = os.environ['GPSD_UNITS']
@@ -801,6 +806,36 @@ def gpsd_units():
     return unspecified
 
 
+class unit_adjustments(object):
+    """Encapsulate adjustments for unit systems."""
+
+    def __init__(self, units=None):
+        "Initialize class unit_adjustments"
+        self.altfactor = gps.METERS_TO_FEET
+        self.altunits = "ft"
+        self.name = "imperial"
+        self.speedfactor = gps.MPS_TO_MPH
+        self.speedunits = "mph"
+        if units is None:
+            units = gpsd_units()
+        if units in (unspecified, imperial, "imperial", "i"):
+            pass
+        elif units in (nautical, "nautical", "n"):
+            self.altfactor = gps.METERS_TO_FEET
+            self.altunits = "ft"
+            self.name = "nautical"
+            self.speedfactor = gps.MPS_TO_KNOTS
+            self.speedunits = "knots"
+        elif units in (metric, "metric", "m"):
+            self.altfactor = 1.0
+            self.altunits = "m"
+            self.name = "metric"
+            self.speedfactor = gps.MPS_TO_KPH
+            self.speedunits = "km/h"
+        else:
+            raise ValueError  # Should never happen
+
+
 # Arguments are in signed decimal latitude and longitude. For example,
 # the location of Montevideo (GF15vc) is: -34.91, -56.21166
 # plagarized from https://ham.stackexchange.com/questions/221
@@ -858,6 +893,7 @@ def maidenhead(dec_lat, dec_lon):
             grid_lon_subsq + grid_lat_subsq +
             grid_lon_extsq + grid_lat_extsq)
 
+
 def __bilinear(lat, lon, table):
     """Return bilinear interpolated data from table"""
 
@@ -869,19 +905,19 @@ def __bilinear(lat, lon, table):
     if _non_finite(lat) or _non_finite(lon):
         return ''
 
-    if math.fabs(lat) > 90 or math.fabs(lon) > 180:
+    if 90 < math.fabs(lat) or 180 < math.fabs(lon):
         return ''
 
     row = int(math.floor((90.0 + lat) / TABLE_SPAN))
     column = int(math.floor((180.0 + lon) / TABLE_SPAN))
 
-    if row < (TABLE_ROWS - 1):
+    if (TABLE_ROWS - 1) > row:
         grid_w = row
         grid_e = row + 1
     else:
         grid_w = row - 1
         grid_e = row
-    if column < (TABLE_COLS - 1):
+    if (TABLE_COLS - 1) > column:
         grid_s = column
         grid_n = column + 1
     else:
@@ -908,12 +944,14 @@ def __bilinear(lat, lon, table):
 
 def mag_var(lat, lon):
     """Return magnetic variation (declination) in degrees.
+
 Given a lat/lon in degrees"""
     return __bilinear(lat, lon, magvar_table)
 
 
 def wgs84_separation(lat, lon):
     """Return MSL-WGS84 geodetic separation in meters.
+
 Given a lat/lon in degrees"""
     return __bilinear(lat, lon, GEOID_DELTA)
 
